import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.ImageIcon;

//===================================================
class WorldObject {
//===================================================
	protected double x, y;
	public WorldObject(double x, double y) { this.x = x; this.y = y; }
	public void setX(double x) { this.x = x; }
	public void setY(double y) { this.y = y; }
	public double getX() { return x; }
	public double getY() { return y; }
	public void draw(Graphics g, int screenX, int screenY) {}
}

//===================================================
class Plane extends WorldObject {
//===================================================
	private ImageIcon spriteLeft;
	private ImageObserver imageObserver;
	public Plane(double x, double y, ImageObserver io) {
		super(x, y);
		imageObserver = io;
		spriteLeft = new ImageIcon("mig21left.png");
	}
	public void draw(Graphics g, int screenX, int screenY) {
		g.drawImage(spriteLeft.getImage(), screenX - spriteLeft.getIconWidth() / 2, screenY - spriteLeft.getIconHeight(), imageObserver);
	}
}

//===================================================
class House extends WorldObject {
//===================================================
	public House(double x, double y) {
		super(x, y);
	}
	public void draw(Graphics g, int screenX, int screenY) {
		g.setColor(new Color(192, 0, 0));
		g.fillRect(screenX - 16, screenY - 24, 32, 12);
		g.setColor(Color.white);
		g.fillRect(screenX - 15, screenY - 12, 30, 12);
	}
}

//===================================================
class PineTree extends WorldObject {
//===================================================
	private int[] tX;
	private int[] tY;
	public PineTree(double x, double y) {
		super(x, y);
		tX = new int[3];
		tY = new int[3];
	}
	public void draw(Graphics g, int screenX, int screenY) {
		g.setColor(new Color(130, 80, 0));
		g.fillRect(screenX, screenY - 5, 2, 5);
		tX[0] = screenX;
		tY[0] = screenY - 35;
		tX[1] = screenX - 5;
		tY[1] = screenY - 5;
		tX[2] = screenX + 5;
		tY[2] = screenY - 5;
		g.setColor(Color.green);
		g.fillPolygon(tX, tY, 3);
	}
}

//===================================================
class OakTree extends WorldObject {
//===================================================
	public OakTree(double x, double y) {
		super(x, y);
	}
	public void draw(Graphics g, int screenX, int screenY) {
		g.setColor(new Color(130, 80, 0));
		g.fillRect(screenX, screenY - 15, 2, 15);
		g.setColor(Color.green);
		g.fillOval(screenX - 5, screenY - 35, 10, 25);
	}
}



//===================================================
public class Mig21Game extends Frame implements KeyListener {
//===================================================
	private double worldLength, planeSpeed;
	private boolean left, right, up, down;
	private WorldObject[] objects;
	private MyCanvas mc;
	private MyThread mt;

	//-----------------------------------------
	class MyThread extends Thread {
	//-----------------------------------------
		public void run() {
			try {
				while (true) {
// For debug purposes:
System.out.print((left)?("<= "):("-- "));
System.out.print((right)?("=> "):("-- "));
System.out.print((up)?("^ "):("| "));
System.out.println(((down)?("v "):("| ")) + "speed:" + planeSpeed + " Y:" + objects[0].getY() + " X:" + (int)objects[0].getX());

					if (left) planeSpeed += 4.0;
					if (right) planeSpeed -= 2.0;
					objects[0].setX(objects[0].getX() + planeSpeed * 0.02);
					while (objects[0].getX() >= worldLength) objects[0].setX(objects[0].getX() - worldLength);
					while (objects[0].getX() < 0.0) objects[0].setX(objects[0].getX() + worldLength);
					if (up) if (objects[0].getY() < 57.0) objects[0].setY(objects[0].getY() + 1.0);
					if (down) if (objects[0].getY() > 0.0) objects[0].setY(objects[0].getY() - 1.0);
					mc.repaint();
					sleep(20);
				}
			} catch (InterruptedException e) {}
		}
	}

	//-----------------------------------------
	class MyCanvas extends Canvas {
	//-----------------------------------------
		private Image offscreenImage;
		private Graphics offscreenGraphics;
		private int screenX, screenY;
		private double objX;
		public MyCanvas() {
			setSize(512, 256);
		}
		public void paint(Graphics g) {
			if (offscreenGraphics == null) {
				offscreenImage = createImage(512, 256);
				offscreenGraphics = offscreenImage.getGraphics();
    			}
			offscreenGraphics.setColor(new Color(192, 192, 255));
			offscreenGraphics.fillRect(0, 0, 512, 256);
			for (int i = 39; i >= 0; i--) {
				objX = objects[0].getX() - objects[i].getX();
				while (objX >= worldLength / 2.0) objX -= worldLength;
				while (objX < -worldLength / 2.0) objX += worldLength;
				screenX = 256 + (int)(objX * 3.0);
				screenY = 192 - (int)(objects[i].getY() * 3.0);
				objects[i].draw(offscreenGraphics, screenX, screenY);
			}
			offscreenGraphics.setColor(new Color(0, 128, 0));
			offscreenGraphics.fillRect(0, 192, 512, 64);
			g.drawImage(offscreenImage, 0, 0, this);
		}
		public void update(Graphics g) {
			paint(g);
		}
	}

	public Mig21Game() {
		super("MIG 21");
		worldLength = 500.0;
		planeSpeed = 80.0;
		objects = new WorldObject[40];
		objects[0] = new Plane(worldLength / 2.0, 20.0, this);  // A repl a nulladik index objektum (ez a megklnbztets mg nem rgja fel a heterogn adaszerkezet szemllett).
		for (int i = 1; i < 40; i++) {
			double rnd = Math.random();
			if (rnd < 0.2) objects[i] = new House((int)(worldLength * Math.random()), 0.0);
			else if (rnd < 0.6) objects[i] = new PineTree((int)(worldLength * Math.random()), 0.0);
			else objects[i] = new OakTree((int)(worldLength * Math.random()), 0.0);
		}
		mc = new MyCanvas();
		setLayout(new BorderLayout());
		add(mc, BorderLayout.CENTER);
		pack();
		setVisible(true);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		addKeyListener(this);
		mc.addKeyListener(this);
		mt = new MyThread();
		mt.start();
	}

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == e.VK_LEFT) left = true;
		if (e.getKeyCode() == e.VK_RIGHT) right = true;
		if (e.getKeyCode() == e.VK_UP) up = true;
		if (e.getKeyCode() == e.VK_DOWN) down = true;
		if (e.getKeyCode() == e.VK_ESCAPE) System.exit(0);
	}
	public void keyReleased(KeyEvent e) {
		if (e.getKeyCode() == e.VK_LEFT) left = false;
		if (e.getKeyCode() == e.VK_RIGHT) right = false;
		if (e.getKeyCode() == e.VK_UP) up = false;
		if (e.getKeyCode() == e.VK_DOWN) down = false;
	}
	public void keyTyped(KeyEvent e) {}

	public static void main(String[] args) {
		new Mig21Game();
	}
}